#ifndef _DECODE_H_
#define _DECODE_H_


// WM_RESET
#define WM_RESET      WM_USER + 0x00F0

// Result
#define ERR_OK				0
#define ERR_GENERAL			-1
#define ERR_INCOMPLETE		-2
#define ERR_BUFFER_FULL		-3

// Type constants

// groups
#define T_GROUP			0
#define T_BIT_GROUP		1
#define T_DATA			2

// strings
#define T_STRING		100
#define T_ERROR			101
#define T_INFO			102

// addresses
#define T_HW_ADDR		200
#define	T_MAC_ADDR		201
#define T_PROTO_ADDR	202
#define T_IPV4_ADDR		203
#define T_IPX_NET		204
#define T_IPX_NODE		205
#define T_IPV6_ADDR		206

// numbers
#define T_NUMBER_BINARY		500
#define T_NUMBER_CHECKSUM	501
#define T_NUMBER_PORT_TCP	502
#define T_NUMBER_PORT_UDP	503
#define T_NUMBER_SIZE		504
#define T_NUMBER_ID			505
#define T_NUMBER_COUNT		506
#define T_NUMBER_TIMESTAMP	507
#define T_NUMBER_VERSION	508
#define T_NUMBER_FLAG		509
#define T_NUMBER_FLOAT		510
#define T_NUMBER_SOCKET_IPX	511
#define T_NUMBER_CHANNEL	512
#define T_NUMBER_SDEC	  598
#define T_NUMBER_DEC		599
#define T_NUMBER_OTHER		600

// date, time, etc
#define T_DATE		900
#define T_TIME		901

// Limits
#define NAME_SIZE	  200
#define VALUE_SIZE	200

// Define formats
#define F_HEX_DEC	  0x01
#define F_DEC_HEX	  0x02
#define F_HEX		    0x03
#define F_DEC		    0x04
#define F_IPV6_HEX  0x05
#define F_IPV6_IPV4 0x06
#define F_SDEC		  0x07

// Define protos
#define PROTO_CLEAR 0
#define PROTO_TFTP  1
#define PROTO_RTP   2

// Define transports
#define TRANSPORT_ANY   0
#define TRANSPORT_TCP   1
#define TRANSPORT_UDP   2
#define TRANSPORT_SAP   3
#define TRANSPORT_ETYPE 4

// Define file formats
#define FRMT_NAIWIN   0
#define FRMT_NETMON   1
#define FRMT_TCPDUMP  2
#define FRMT_OBSERVER 3
#define FRMT_WILDPKTS 4
#define FRMT_PCAPNG   5
#define FRMT_MASK         0x0FFFF
#define FRMT_WITH_RESUME  0x10000
#define FRMT_RESUME       0x20000

// Define frame types
#define FRAME_ETHERNET  0
#define FRAME_802_11    1
#define FRAME_TOKENRING 2

#pragma pack(push, 1)
struct format {
	int type;
	int output;
};

struct Tree {
  int level;
  int type;
  int offset;
  int size;
  char name[NAME_SIZE + 1];
  char value[VALUE_SIZE + 1];
};

struct help {
  char  name[256];
  char  path[256];
};

struct protocol {
  int   proto_id;
  char  name[NAME_SIZE + 1];
};
#pragma pack(pop)


struct conversation {
  int proto;
  unsigned short src_port;
  unsigned short dst_port;
  unsigned long src_ip;
  unsigned long dst_ip;

  conversation *next;
};

// UDP conversations
extern conversation *udp_conv;

// Define formats for types
extern format c_fmt[];

// Function for putting new record
void PutLine(Tree *p, int level, int type, int offset, int size, char *name, char *value);

unsigned short __inline fcd_ntohs(unsigned short n) {
	return static_cast<unsigned short>((n & 0xFF) << 8 | (n>> 8));
}

unsigned short __inline cntohs(unsigned short n, bool condition) {
  if (condition) {
	  return static_cast<unsigned short>((n & 0xFF) << 8 | (n>> 8));
  } else {
    return n;
  }
}

unsigned short __inline ntohsp(unsigned char *p) {
	return static_cast<unsigned short>((p[0] << 8) | p[1]);
}

unsigned short __inline cntohsp(unsigned char *p, bool condition) {
	if (condition) {
		return static_cast<unsigned short>((p[0] << 8) | p[1]);
	} else {
		return static_cast<unsigned short>((p[1] << 8) | p[0]);
	}
}


unsigned int __inline ntoh3p(unsigned char *p) {
	return static_cast<unsigned long>((p[0] << 16) | (p[1] << 8) | p[2]);
}

unsigned int __inline fcd_ntohl(unsigned int n) {
	unsigned char *p = reinterpret_cast<unsigned char *>(&n);
	return static_cast<unsigned long>(p[0] << 24) | (p[1] << 16) | (p[2] << 8) | p[3];
}

unsigned long __inline cntohl(unsigned int n, bool condition) {
 	unsigned char *p = reinterpret_cast<unsigned char *>(&n);

  if (condition) {
	  return static_cast<unsigned long>(p[0] << 24) | (p[1] << 16) | (p[2] << 8) | p[3];
  } else {
    return n;
  }
}

unsigned int __inline ntohlp(unsigned char *p) {
	return static_cast<unsigned long>(p[0] << 24) | (p[1] << 16) | (p[2] << 8) | p[3];
}

unsigned __int64 __inline ntoh64(unsigned __int64 n) {
	unsigned char *p = reinterpret_cast<unsigned char *>(&n);
	return (static_cast<unsigned __int64>(p[0]) << 56) | (static_cast<unsigned __int64>(p[1]) << 48) | (static_cast<unsigned __int64>(p[2]) << 40) | (static_cast<unsigned __int64>(p[3]) << 32) | (static_cast<unsigned __int64>(p[4]) << 24) | (static_cast<unsigned __int64>(p[5]) << 16) | (static_cast<unsigned __int64>(p[6]) << 8) | static_cast<unsigned __int64>(p[7]);
}

unsigned __int64 __inline ntoh64p(unsigned char *p) {
  return (static_cast<unsigned __int64>(p[0]) << 56) | (static_cast<unsigned __int64>(p[1]) << 48) | (static_cast<unsigned __int64>(p[2]) << 40) | (static_cast<unsigned __int64>(p[3]) << 32) | (static_cast<unsigned __int64>(p[4]) << 24) | (static_cast<unsigned __int64>(p[5]) << 16) | (static_cast<unsigned __int64>(p[6]) << 8) | static_cast<unsigned __int64>(p[7]);
}

unsigned __int64 __inline ntoh6p(unsigned char *p) {
	return (static_cast<unsigned __int64>(p[0]) << 40) | (static_cast<unsigned __int64>(p[1]) << 32) | (static_cast<unsigned __int64>(p[2]) << 24) | (static_cast<unsigned __int64>(p[3]) << 16) | (static_cast<unsigned __int64>(p[4]) << 8) | static_cast<unsigned __int64>(p[5]);
}

unsigned __int64 __inline ntoh5p(unsigned char *p) {
	return (static_cast<unsigned __int64>(p[0]) << 32) | (static_cast<unsigned __int64>(p[1]) << 24) | (static_cast<unsigned __int64>(p[2]) << 16) | (static_cast<unsigned __int64>(p[3]) << 8) | static_cast<unsigned __int64>(p[4]);
}

unsigned __int64 __inline htoh64p(unsigned char *p) {
  return (static_cast<unsigned __int64>(p[7]) << 56) | (static_cast<unsigned __int64>(p[6]) << 48) | (static_cast<unsigned __int64>(p[5]) << 40) | (static_cast<unsigned __int64>(p[4]) << 32) | (static_cast<unsigned __int64>(p[3]) << 24) | (static_cast<unsigned __int64>(p[2]) << 16) | (static_cast<unsigned __int64>(p[1]) << 8) | static_cast<unsigned __int64>(p[0]);
}

// main function - decode dll
extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
	int UnmangleW(Tree *buffer, int buf_size, unsigned char *packet, int pack_size, __int64 id, unsigned short hour, unsigned short min, unsigned short sec, unsigned short msec);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
	void UnmangleW_(char *summary, unsigned char *packet, int pack_size, __int64 id);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
	int SetDataA(Tree *buffer, int buf_size, unsigned char *packet, int pack_size, __int64 id, unsigned short hour, unsigned short min, unsigned short sec, unsigned short msec);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
	void SetDataA_(char *summary, unsigned char *packet, int pack_size, __int64 id);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
	int SetDataW(Tree *buffer, int buf_size, unsigned char *packet, int pack_size, __int64 id, unsigned short hour, unsigned short min, unsigned short sec, unsigned short msec, unsigned char DecryptionSucessful);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
	void SetDataW_(char *summary, unsigned char *packet, int pack_size, __int64 id, unsigned char DecryptionSucessful);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		bool GetData(unsigned char *rawData, unsigned short len, unsigned char *correctedData);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		int GetDataA(help hlp[], int num);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		void GetDataW(HWND hwid);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		void CheckData(format *fmt);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		void CheckDataA(__int64 ids[], int count);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		int ResetData(char *fn, unsigned char *buf, unsigned int size, unsigned int frmt, unsigned int *frame_type);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		int ResetDataQ(char *fn, unsigned char *buf, unsigned int size, unsigned int frmt, unsigned int frame_type, int phase, int total_packets);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		bool UncheckData(unsigned char *Data, unsigned short len);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		bool SetMasterA(int proto, unsigned short src_port, unsigned short dst_port, unsigned long src_ip, unsigned long dst_ip);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		void SetSlave(int *ports, int size);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		int InitializeDataA(int transport, protocol *proto_list, int size);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		void InitializeDataW(int transport, int proto_id, int port);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		void SetForced(int doset);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		bool ParseDataA(char *str, int str_len);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
    void ParseDataW(HWND ahandle, bool create);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
    bool ParseData(int transport, int proto_id, HWND window);
}

extern "C" {
#ifdef _DECODE_EXPORT_
  __declspec(dllexport)
#endif //_DECODE_EXPORT_
    void P1(unsigned short *P1K, const unsigned char *TK, const unsigned char *TA, unsigned long IV32);
}

extern "C" {
#ifdef _DECODE_EXPORT_
  __declspec(dllexport)
#endif //_DECODE_EXPORT_
    void P2(unsigned char *RC4KEY, const unsigned char *TK, const unsigned short *P1K, unsigned short IV16);
}

extern "C" {
#ifdef _DECODE_EXPORT_
  __declspec(dllexport)
#endif //_DECODE_EXPORT_
  void WP1(unsigned char *key);
}

extern "C" {
#ifdef _DECODE_EXPORT_
  __declspec(dllexport)
#endif //_DECODE_EXPORT_
  void WP2(char *passphrase);
}

extern "C" {
#ifdef _DECODE_EXPORT_
  __declspec(dllexport)
#endif //_DECODE_EXPORT_
  void WP3();
}

extern "C" {
#ifdef _DECODE_EXPORT_
  __declspec(dllexport)
#endif //_DECODE_EXPORT_
  int WP4(void *Buffer, int numrecs, HWND WinHandle);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
	void PG1(void *func);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		int PG2(void *buf, int num);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		int PG3(void *buf, int num);
}

extern "C" {
#ifdef _DECODE_EXPORT_
  __declspec(dllexport)
#endif //_DECODE_EXPORT_
    void PG4(void *func);
}

extern "C" {
#ifdef _DECODE_EXPORT_
	__declspec(dllexport)
#endif //_DECODE_EXPORT_
		void PG5();
}

#endif //_DECODE_H_
